/* ************************************************************************** */
/*                                                                            */
/*                                                        :::      ::::::::   */
/*   main.c                                             :+:      :+:    :+:   */
/*                                                    +:+ +:+         +:+     */
/*   By: tmaze <tmaze@student.42.fr>                +#+  +:+       +#+        */
/*                                                +#+#+#+#+#+   +#+           */
/*   Created: 2019/09/19 17:08:46 by tmaze             #+#    #+#             */
/*   Updated: 2020/02/07 15:11:58 by tmaze            ###   ########.fr       */
/*                                                                            */
/* ************************************************************************** */

#include "minishell.h"

char	*inc_shlvl(char **shlvl)
{
	char	*ret;

	if ((ret = ft_itoa(ft_atoi(*shlvl) + 1)) != NULL)
	{
		ft_strdel(shlvl);
		*shlvl = ret;
	}
	return (ret);
}

t_env	*env2lst(char **env)
{
	t_env	*ret;
	t_env	*new;
	int		i;

	i = 0;
	ret = NULL;
	while (env && env[i])
	{
		if ((new = ft_envnew(env[i])) != NULL)
		{
			if (ft_strequ(new->key, "SHLVL") && ft_str_is_numeric(new->val))
				inc_shlvl(&new->val);
			ft_envaddend(&ret, new);
		}
		else
		{
			ft_envdelelem(&new);
			ft_envdel(&ret);
			return (NULL);
		}
		i++;
	}
	if (i == 0 && ret == NULL && (ret = def_env()) == NULL)
		ft_printf("Env: Memory Error\n");
	return (ret);
}

void	check_exec(char **argv, t_env *env)
{
	int					i;
	static t_builtin	built[S_BIN] = {{"env", &cmd_env}, {"cd", &cmd_cd}
								, {"setenv", &cmd_setenv}, {"echo", &cmd_echo}
								, {"unsetenv", &cmd_unsetenv}};

	i = -1;
	while (++i < S_BIN)
		if (ft_strequ(argv[0], built[i].cmd))
		{
			built[i].f(argv, &env);
			break ;
		}
	if (i == S_BIN)
		exec_cmd(argv, &env);
}

int		cleanup(char **argv, t_env *env)
{
	ft_del_words_tables(&argv);
	ft_envdel(&env);
	return (0);
}

int		main(void)
{
	extern char	**environ;
	char		**argv;
	t_env		*env;
	char		*cmd;

	if ((env = env2lst(environ)) == NULL)
		return (2);
	while (1)
	{
		ft_printf("%s$>%s ", FT_COLOR_GREEN, FT_RESET);
		if (ft_getline(&cmd) >= 0 && cmd != NULL
			&& (argv = ft_strsplit(cmd, ' ')) != NULL && !ft_strdel_null(&cmd))
		{
			if ((res_ext(argv, env)) != NULL)
			{
				if (ft_strequ(argv[0], "exit"))
					return (cleanup(argv, env));
				check_exec(argv, env);
			}
		}
		ft_strdel(&cmd);
		ft_del_words_tables(&argv);
	}
	return (0);
}
